SystemOrganization addCategory: #'GT-Tests-Examples'!

Object subclass: #GTInspectorExamplesManualTest
	instanceVariableNames: 'allFailures allClasses allTestedClasses allUntestedClasses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Tests-Examples'!

!GTInspectorExamplesManualTest commentStamp: 'StefanReichhart 4/26/2016 21:38' prior: 0!
I am a test for manually checking that the inspector presentations work on all the examples objects in the system. I however exclude tests from some classes that can be problematic.

I maintain the list of classes for which examples were tested in an external file, so if the image crashed I can resume checking.

#run is the main entry point to the test
!

!GTInspectorExamplesManualTest class methodsFor: 'public' stamp: 'StefanReichhart 11/14/2015 14:45'!
reset
	FileSystem workingDirectory allFiles 
		select: [ :each | each basename beginsWith: 'gtExamples' ]
		thenDo: [ :each | each delete ]! !

!GTInspectorExamplesManualTest methodsFor: 'private' stamp: 'StefanReichhart 11/14/2015 14:52'!
basicRunClasses
	 self runClasses: allUntestedClasses ! !

!GTInspectorExamplesManualTest methodsFor: 'private' stamp: 'StefanReichhart 11/14/2015 13:44'!
basicRunExample: anExample
	self checkAllPresentationsOf: anExample onError: [ :aPragma :anException | 
		allFailures add: anExample method.
		self printException: anException forExample: anExample withPragma: aPragma ] ! !

!GTInspectorExamplesManualTest methodsFor: 'private' stamp: 'StefanReichhart 11/14/2015 14:49'!
basicRunFailures
	" self new runFailures "

	(allFailures := self readFailuresFromFile: (FileSystem workingDirectory / 'gtExamplesAllFailures.txt')) 
		do: [ :each |
			each gtExamples do: [ :example |
				self runExample: example ] ]
		displayingProgress: 'Running ', allFailures size asString, ' failures ...'! !

!GTInspectorExamplesManualTest methodsFor: 'private' stamp: 'AndreiChis 1/19/2016 14:54'!
checkAllPresentationsOf: exampleToTest onError: aBlock
	| browser window |
	(Pragma allNamed: #gtInspectorPresentationOrder: from: exampleToTest class to: ProtoObject) do: [ :eachPragma | 
		browser := GLMWrapper new.
		[	browser
				show: [ :composite | 
					(eachPragma selector findTokens: $:) size = 1
						ifTrue: [ exampleToTest perform: eachPragma selector with: composite ].
					(eachPragma selector findTokens: $:) size = 2
						ifTrue: [ exampleToTest perform: eachPragma selector with: composite with: browser ] ].
			window := browser openOn: exampleToTest ] 
				on: Error 
				do: [ :exception |
					window ifNotNil: [ window delete ].
					aBlock value: eachPragma value: exception ].
		window ifNotNil: [ window delete ] ]! !

!GTInspectorExamplesManualTest methodsFor: 'private' stamp: 'BorisSpasojevic 1/20/2016 13:34'!
examplesForClass: aClass
	^ aClass gtExamples! !

!GTInspectorExamplesManualTest methodsFor: 'private' stamp: 'BorisSpasojevic 1/20/2016 13:35'!
filenameFor: anExample
	^ ((('gtExamplesFailure_', anExample method gtDisplayString		 
			copyReplaceAll: '>' with: '_') 
			copyReplaceAll: ':' with: '.') 
			copyReplaceAll: ' ' with: '') 
			copyReplaceAll: '#' with: ''! !

!GTInspectorExamplesManualTest methodsFor: 'initialization' stamp: 'StefanReichhart 11/14/2015 14:49'!
initialize
	| fileWithAllClasses fileWithAllTestedClasses |
	super initialize.
		
	" prepare all classes "
	allClasses := {}.
	fileWithAllClasses := FileSystem workingDirectory / 'gtExamplesAllClasses.txt'.
	fileWithAllClasses exists ifTrue: [ 
		allClasses := self readClassesFromFile: fileWithAllClasses ].
	allClasses isEmptyOrNil ifTrue: [
		| streamWithAllClasses |
		allClasses := SystemNavigation default allClassesAndTraits sort: [ :a :b | a name <= b name ].
		streamWithAllClasses := fileWithAllClasses writeStream.
		allClasses do: [ :each | streamWithAllClasses nextPutAll: each name; cr. ].
		streamWithAllClasses flush; close. ].

	" prepare tested classes "
	allTestedClasses := {}.
	fileWithAllTestedClasses := FileSystem workingDirectory / 'gtExamplesAllTestedClasses.txt'.
	fileWithAllTestedClasses exists ifTrue: [ 
		allTestedClasses := self readClassesFromFile: fileWithAllTestedClasses ].
	
	" prepare untested classes "
	allUntestedClasses := (allClasses difference: allTestedClasses) sort: [ :a :b | a name <= b name ].
	
	" prepare failures "
	allFailures := self readFailuresFromFile: (FileSystem workingDirectory / 'gtExampleAllFailures.txt').! !

!GTInspectorExamplesManualTest methodsFor: 'private' stamp: 'StefanReichhart 11/14/2015 14:50'!
printException: anException forExample: anExample withPragma: aPragma
	(FileSystem workingDirectory / 'gtExamplesAllFailures.txt') writeStream 
		setToEnd; cr;
		nextPutAll: anExample method gtDisplayString; cr;
		flush;
		close.
	(FileSystem workingDirectory / (self filenameFor: anExample)) writeStream 
		truncate;
		nextPutAll: anExample gtDisplayString; cr;
		nextPutAll: aPragma gtDisplayString; cr;
		nextPutAll: anException asString; cr;
		nextPutAll: '--------------------------------------------------------------------------------'; cr;
		nextPutAll: (String streamContents: [ :stream | 
							anException signalerContext ifNotNil: [ :context | 
								context debugStackOn: stream ] ] );
		flush;
		close! !

!GTInspectorExamplesManualTest methodsFor: 'private' stamp: 'StefanReichhart 11/14/2015 13:56'!
readClassesFromFile: aFile
	aFile exists ifFalse: [ ^ OrderedCollection new ].
	^ ((aFile readStream contents splitOn: String cr) 
		collect: [ :each | [ Smalltalk classNamed: each trimBoth ] on: Error do: [ nil ] ]) 
		reject: [ :each | each isNil ]! !

!GTInspectorExamplesManualTest methodsFor: 'private' stamp: 'StefanReichhart 8/18/2016 21:47'!
readFailuresFromFile: aFile
	aFile exists ifFalse: [ ^ OrderedCollection new ].
	^ (((aFile readStream contents splitOn: String cr)
		collect: [ :each | [ Smalltalk compiler evaluate: each trimBoth ] on: Error do: [ nil ] ])
		reject: [ :each | each isNil ]) asSet asOrderedCollection! !

!GTInspectorExamplesManualTest methodsFor: 'public' stamp: 'StefanReichhart 11/14/2015 14:52'!
run
	" self new run "
	
	self run: [ self basicRunClasses ]! !

!GTInspectorExamplesManualTest methodsFor: 'private' stamp: 'StefanReichhart 11/14/2015 13:58'!
run: aBlock
	
	[ GTExampleOrganizer instance stop. aBlock value ]
		ensure: [ GTExampleOrganizer instance start ].
	
	self verify! !

!GTInspectorExamplesManualTest methodsFor: 'private' stamp: 'BorisSpasojevic 1/20/2016 13:34'!
runClasses: aCollectionOfClasses
	aCollectionOfClasses
		do:
			[ :class | 
			(self examplesForClass: class) do: [ :example | self runExample: example ] displayingProgress: class name.
			(FileSystem workingDirectory / 'gtExamplesAllTestedClasses.txt') writeStream
				setToEnd;
				cr;
				nextPutAll: class name;
				flush;
				close ]
		displayingProgress: 'Running examples of ' , aCollectionOfClasses size asString , ' classes ...'! !

!GTInspectorExamplesManualTest methodsFor: 'private' stamp: 'StefanReichhart 11/14/2015 13:44'!
runExample: anExample
	[ self basicRunExample: anExample ]
		on: Error
		do: [ :exception | 
			allFailures add: anExample method.
			self printException: exception forExample: anExample withPragma: nil ] ! !

!GTInspectorExamplesManualTest methodsFor: 'public' stamp: 'StefanReichhart 11/14/2015 13:58'!
runFailures
	" self new runFailures "
	
	self run: [ self basicRunFailures ]! !

!GTInspectorExamplesManualTest methodsFor: 'private' stamp: 'StefanReichhart 11/14/2015 13:32'!
verify
	self assert: allFailures isEmptyOrNil description: (String streamContents: [ :stream |
		allFailures do: [ :each | 
			stream nextPutAll: each gtDisplayString; cr ] ] )! !

GTExampleTestCase subclass: #GTToolkitExamplesTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Tests-Examples'!

!GTToolkitExamplesTests methodsFor: 'running' stamp: 'StefanReichhart 8/19/2016 12:50'!
setUp
	super setUp.
	
	GTExampleOrganizer reset; stop! !

!GTToolkitExamplesTests methodsFor: 'running' stamp: 'StefanReichhart 8/19/2016 12:49'!
tearDown
	super tearDown.
	
	GTExampleOrganizer reset; start! !

!GTToolkitExamplesTests methodsFor: 'tests' stamp: 'StefanReichhart 9/26/2015 11:32'!
testExamplesDebugger
	self runExampleContainers: (self packages: { 'GT-Debugger*' })! !

!GTToolkitExamplesTests methodsFor: 'tests' stamp: 'StefanReichhart 9/26/2015 10:52'!
testExamplesInspector
	self runExampleContainers: (self packages: { 'GT-Inspector*' })! !

!GTToolkitExamplesTests methodsFor: 'tests' stamp: 'StefanReichhart 9/26/2015 10:52'!
testExamplesPlayground
	self runExampleContainers: (self packages: { 'GT-Playground*' })! !

!GTToolkitExamplesTests methodsFor: 'tests' stamp: 'StefanReichhart 9/26/2015 10:52'!
testExamplesSpotter
	self runExampleContainers: (self packages: { 'GT-Spotter*' })! !

TestCase subclass: #GTExampleMethodResolverTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Tests-Examples'!

!GTExampleMethodResolverTest methodsFor: 'utils' stamp: 'StefanReichhart 2/18/2017 19:24'!
assert: aGTExampleMethod class: aClass classNamed: aClassName meta: aBoolean selector: aSelector

	self assert: aGTExampleMethod methodClass == aClass.
	self assert: aGTExampleMethod methodClassName = aClassName.
	self assert: aGTExampleMethod meta = aBoolean.
	self assert: aGTExampleMethod selector = aSelector.
	self assert: aGTExampleMethod selector isSymbol.
	
	self assert: aGTExampleMethod exists.
	
	self shouldnt: [ aGTExampleMethod resetStub ] raise: Exception.
	self assert: (aGTExampleMethod instVarNamed: 'methodClass') isNil.
	
	self assert: aGTExampleMethod methodClass == aClass.
	self assert: aGTExampleMethod methodClassName = aClassName.
	self assert: aGTExampleMethod meta = aBoolean.
	self assert: aGTExampleMethod selector = aSelector.
	self assert: aGTExampleMethod selector isSymbol.
	
	self assert: aGTExampleMethod exists.! !

!GTExampleMethodResolverTest methodsFor: 'tests-convenience' stamp: 'StefanReichhart 2/18/2017 20:16'!
testResolveByName
	| method |
	method := GTExampleMethodResolver resolve: GTExampleMethod example: nil className: #GTExampleMethodResolver meta: false selector: #selector.
	
	self assert: method class: GTExampleMethodResolver classNamed: #GTExampleMethodResolver meta: false selector: #selector.
	self assert: method class: GTExampleMethodResolver classNamed: 'GTExampleMethodResolver' meta: false selector: #selector.
	
	method := GTExampleMethodResolver resolve: GTExampleMethod example: nil className: 'GTExampleMethodResolver' meta: false selector: 'selector'.
	
	self assert: method class: GTExampleMethodResolver classNamed: #GTExampleMethodResolver meta: false selector: #selector.
	self assert: method class: GTExampleMethodResolver classNamed: 'GTExampleMethodResolver' meta: false selector: #selector.! !

!GTExampleMethodResolverTest methodsFor: 'tests-convenience' stamp: 'StefanReichhart 2/19/2017 15:46'!
testResolveMetaByName
	| method |
	method := GTExampleMethodResolver resolve: GTExampleMethod example: nil className: #GTExampleMethodResolver meta: true selector: #resolve:example:selector:.
	
	self assert: method class: GTExampleMethodResolver class classNamed: 'GTExampleMethodResolver class' meta: true selector: #resolve:example:selector:.
	self assert: method class: GTExampleMethodResolver class classNamed: 'GTExampleMethodResolver class' meta: true selector: #resolve:example:selector:.
	
	method := GTExampleMethodResolver resolve: GTExampleMethod example: nil className: 'GTExampleMethodResolver' meta: true selector: #resolve:example:selector:.
	
	self assert: method class: GTExampleMethodResolver class classNamed: 'GTExampleMethodResolver class' meta: true selector: #resolve:example:selector:.
	self assert: method class: GTExampleMethodResolver class classNamed: 'GTExampleMethodResolver class' meta: true selector: #resolve:example:selector:.! !

!GTExampleMethodResolverTest methodsFor: 'tests-convenience' stamp: 'StefanReichhart 2/18/2017 20:17'!
testResolveMetaByNameWithSystemCompatibility
	| method |
	method := GTExampleMethodResolver resolve: GTExampleMethod example: nil className: 'GTExampleMethodResolver class' meta: true selector: #resolve:example:selector:.
	
	self assert: method class: GTExampleMethodResolver class classNamed: 'GTExampleMethodResolver class' meta: true selector: #resolve:example:selector:.
	self assert: method class: GTExampleMethodResolver class classNamed: 'GTExampleMethodResolver class' meta: true selector: #resolve:example:selector:.
	
	method := GTExampleMethodResolver resolve: GTExampleMethod example: nil className: 'GTExampleMethodResolver class' meta: false selector: 'resolve:example:selector:'.
	
	self assert: method class: GTExampleMethodResolver class classNamed: 'GTExampleMethodResolver class' meta: true selector: #resolve:example:selector:.
	self assert: method class: GTExampleMethodResolver class classNamed: 'GTExampleMethodResolver class' meta: true selector: #resolve:example:selector:.! !

TestCase subclass: #GTExampleMethodTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Tests-Examples'!

!GTExampleMethodTest methodsFor: 'utils' stamp: 'StefanReichhart 2/18/2017 18:57'!
assert: aGTExampleMethod class: aClass className: aName meta: meta selector: aSelector 

	self assert: aGTExampleMethod methodClass == aClass.
	self assert: aGTExampleMethod methodClassName = aName.
	self assert: aGTExampleMethod meta = meta.
	self assert: aGTExampleMethod selector = aSelector.
	
	self assert: aGTExampleMethod exists.
	self assert: aGTExampleMethod isValid.
	
	self shouldnt: [ aGTExampleMethod resetStub ] raise: Exception.
	self assert: (aGTExampleMethod instVarNamed: 'methodClass') isNil.
	
	self assert: aGTExampleMethod methodClass == aClass.
	self assert: aGTExampleMethod methodClassName = aName.
	self assert: aGTExampleMethod meta = meta.
	self assert: aGTExampleMethod selector = aSelector.
	
	self assert: aGTExampleMethod exists.
	self assert: aGTExampleMethod isValid.! !

!GTExampleMethodTest methodsFor: 'tests' stamp: 'StefanReichhart 2/18/2017 18:58'!
testFromClassNamedSelector
	| exampleMethod |
	exampleMethod := GTExampleMethod fromClassNamed: #GTExampleMethod selector: #isValid.
	self assert: exampleMethod class: GTExampleMethod className: #GTExampleMethod meta: false selector: #isValid.
	
	exampleMethod := GTExampleMethod fromClassNamed: 'GTExampleMethod' selector: #isValid.
	self assert: exampleMethod class: GTExampleMethod className: #GTExampleMethod meta: false selector: #isValid.
	
	exampleMethod := GTExampleMethod fromClassNamed: #GTExampleMethod selector: #isValid.
	self assert: exampleMethod class: GTExampleMethod className: 'GTExampleMethod' meta: false selector: #isValid.! !

!GTExampleMethodTest methodsFor: 'tests' stamp: 'StefanReichhart 2/18/2017 18:58'!
testFromClassSelector
	| exampleMethod |
	exampleMethod := GTExampleMethod fromClass: GTExampleMethod selector: #hasExample.
	self assert: exampleMethod class: GTExampleMethod className: #GTExampleMethod meta: false selector: #hasExample.
	self assert: exampleMethod class: GTExampleMethod className: 'GTExampleMethod' meta: false selector: #hasExample.! !

TestCase subclass: #GTExamplesFinalReleaseTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Tests-Examples'!

!GTExamplesFinalReleaseTest methodsFor: 'running' stamp: 'StefanReichhart 8/28/2016 15:59'!
setUp
	super setUp.
	
	GTExampleOrganizer reset; stop.
	Smalltalk garbageCollect! !

!GTExamplesFinalReleaseTest methodsFor: 'running' stamp: 'StefanReichhart 8/28/2016 15:59'!
tearDown
	super tearDown.
	
	GTExampleOrganizer reset; start.
	Smalltalk garbageCollect! !

!GTExamplesFinalReleaseTest methodsFor: 'tests' stamp: 'StefanReichhart 8/28/2016 14:42'!
testNoExampleInstances

	self assert: GTExample allSubInstances size equals: 0.
	self assert: GTExampleResult allSubInstances size equals: 0.
	! !

TestCase subclass: #GTExamplesReleaseTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Tests-Examples'!

!GTExamplesReleaseTest methodsFor: 'private' stamp: 'StefanReichhart 9/14/2015 21:18'!
assertIncludesPackages: includedPackageNames excludedPackages: excludedPackageNames ignoredClasses: ignoredClasses pragmas: pragmas examples: examples description: aString
	| packages missingExamples missingExamplesWithoutIgnoredClasses obsoleteIgnoredClasses |
	
	" gather all packages that should be checked "
	packages := self includesPackages: includedPackageNames excludedPackages: excludedPackageNames.
	
	" check the given packages and filter out some classes "
	missingExamples := self packages: packages pragmas: pragmas examples: examples.
	missingExamplesWithoutIgnoredClasses := self missingExamples: missingExamples withoutIgnoredClasses: ignoredClasses.
	
	" check for missing examples - ignore some classes "
	self assert: missingExamplesWithoutIgnoredClasses isEmpty 
		description: [ 
			String streamContents: [ :stream |
				stream nextPutAll: (aString ifNil: [ 'Found missing examples in' ]); nextPutAll: ': '.
				(missingExamplesWithoutIgnoredClasses keys sorted: [ :a :b | a name < b name ]) printOn: stream ] ].
	
	" check for ignored examples which are provided - those should no longer be ignored "
	obsoleteIgnoredClasses := (ignoredClasses difference: missingExamples keys) reject: [ :each | each isNil ].
	self assert: obsoleteIgnoredClasses isEmpty 
		description: [ 
			String streamContents: [ :stream |
				stream nextPutAll: 'Examples found. Test should no longer ignore these classes: '.
				(obsoleteIgnoredClasses sorted: [ :a :b | a name < b name ]) printOn: stream ] ]! !

!GTExamplesReleaseTest methodsFor: 'private' stamp: 'StefanReichhart 3/26/2015 23:31'!
includesPackages: includedPackageNames excludedPackages: excludedPackageNames
	| packages |
	packages := OrderedCollection new.
	(RPackageOrganizer default packages 
		sorted: [ :a :b | a packageName < b packageName ]) 
		do: [ :package |
			((includedPackageNames isEmptyOrNil
				or: [ includedPackageNames anySatisfy: [ :packageName | package packageName asLowercase beginsWith: packageName asLowercase ] ])
					and: [ excludedPackageNames isEmptyOrNil
						or: [ excludedPackageNames noneSatisfy: [ :packageName | package packageName asLowercase beginsWith: packageName asLowercase ] ] ])
							ifTrue: [ packages add: package ] ].
	^ packages! !

!GTExamplesReleaseTest methodsFor: 'private' stamp: 'StefanReichhart 3/26/2015 23:41'!
missingExamples: missingExamples withoutIgnoredClasses: classes
	| missingExamplesWithoutIgnoredClasses |
	classes isEmptyOrNil ifTrue: [ ^ missingExamples ].
	missingExamplesWithoutIgnoredClasses := missingExamples copy.
	classes do: [ :class | missingExamplesWithoutIgnoredClasses removeKey: class ifAbsent: [  ] ].
	^ missingExamplesWithoutIgnoredClasses! !

!GTExamplesReleaseTest methodsFor: 'private' stamp: 'StefanReichhart 8/19/2016 20:54'!
packages: packages pragmas: pragmasRequiringExamples examples: examplePragmas
	| missingExamples |
	missingExamples := Dictionary new.
	packages do: [ :package |
		package classes do: [ :class |
			(self shouldDefineExamples: class)  ifTrue: [ " we are only interested in classes "
				| methodsRequiringExamples |
				methodsRequiringExamples := pragmasRequiringExamples flatCollect: [ :each | Pragma allNamed: each in: class ].
				methodsRequiringExamples ifNotEmpty: [ " example required "
					| exampleMethods |
					exampleMethods := examplePragmas flatCollect: [ :each | Pragma allNamed: each in: class theMetaClass ].
					exampleMethods ifEmpty: [ " something is missing - check which package would require the example "
						(methodsRequiringExamples anySatisfy: [ :pragma | (packages includes: pragma method package) ]) ifTrue: [ 
							missingExamples 
								at: class 
								put: (methodsRequiringExamples collect: [ :pragma | pragma method ]) ] ] ] ] ] ].
	^ missingExamples! !

!GTExamplesReleaseTest methodsFor: 'private' stamp: 'StefanReichhart 8/19/2016 21:02'!
shouldDefineExamples: class
	^ (class theNonMetaClass = class)
		and: [ class isTrait not 
			and: [ (class theNonMetaClass name includesSubstring: 'abstract' caseSensitive: false) not 
				and: [ (class theNonMetaClass name includesSubstring: 'mock' caseSensitive: false) not ] ] ]! !

!GTExamplesReleaseTest methodsFor: 'tests' stamp: 'StefanReichhart 8/19/2016 21:01'!
testAllMissingExternalExamples
	<toBeDeprecated: #gtExample:>
	self assertIncludesPackages: nil
		excludedPackages: { 'Glamour-'. 'GT-'. 'Moose-'. 'Petit'. 'Renraku'. 'Slot'. 'Opal'. 'Roassal' }
		ignoredClasses: {
			Behavior. "Has abstract methods"
		 }
		pragmas: #( #spotterOrder: #gtInspectorPresentationOrder: )
		examples: #( #gtExample #gtExample: )
		description: 'All missing external examples'! !

!GTExamplesReleaseTest methodsFor: 'tests' stamp: 'PavelKrivanek 4/21/2017 14:03'!
testAllMissingInternalExamples
	<toBeDeprecated: #gtExample:>
	self assertIncludesPackages: { 'Glamour-'. 'GT-'. 'Moose-'. }
		excludedPackages: nil
		ignoredClasses: {
			ComposableModel. 
			GTExample. " skeleton object "
			GTExampleMethodAfter. " would require mocks "
			GTEventCollectors. 
			GTEventRecorder. 
			MetacelloVersionLoadDirective. " corrupt package detection "
			MorphicTreeNodeAdapter. " corrupt package detection "
			Slot. " corrupt package detection "
			ZnMessage. " corrupt package detection "
			GTExampleIcon. "abstract"
			GTExampleSubject. "abstract"
			CompiledCode. "abstract"
		}
		pragmas: #( #spotterOrder: #gtInspectorPresentationOrder: )
		examples: #( #gtExample #gtExample: )
		description: 'All missing internal examples'! !

!GTExamplesReleaseTest methodsFor: 'tests' stamp: 'PavelKrivanek 4/21/2017 15:10'!
testMissingGTInspectorExamples
	<toBeDeprecated: #gtExample:>
	self assertIncludesPackages: { 'GT-Inspector' }
		excludedPackages: nil
		ignoredClasses: {
			ComposableModel. " abstract superclass "
			MetacelloVersionLoadDirective. " corrupt package detection "
			MorphicTreeNodeAdapter. " corrupt package detection "
			ZnMessage. " abstract factory & superclass / subclasses provide examples "
			CompiledCode. "abstract"
		}
		pragmas: #( #gtInspectorPresentationOrder: )
		examples: #( #gtExample #gtExample: )
		description: 'Missing examples for GTInspector'! !

!GTExamplesReleaseTest methodsFor: 'tests' stamp: 'StefanReichhart 8/19/2016 21:00'!
testMissingGTSpotterExamples
	<toBeDeprecated: #gtExample:>
	self assertIncludesPackages: { 'GT-Spotter' }
		excludedPackages: nil
		ignoredClasses: { 
			Slot. " abstract "
		}
		pragmas: #( #spotterOrder: )
		examples: #( #gtExample #gtExample: )
		description: 'Missing examples for GTSpotter'! !

!GTExamplesReleaseTest methodsFor: 'tests' stamp: 'StefanReichhart 3/27/2015 20:06'!
testMissingTests
	self flag: 'maybe enhance the following abstract classes (generic extension may work for most/all subclasses). Most/all subclasses would be examples. Intermediate classes are not tagged as abstract - not distinguishable - #allSubclasses doesn^t work. Announcer. Canvas. ComposableModel OCAbstractScope. RTView. TRCanvas. IRInstruction. Slot. Slot/Inspector is buggy'.
	self flag: 'find a solution to provide examples for TClass + MooseModel but still have usable tests / examples in both classes will slow down tests tremendously'! !

TestCase subclass: #GTInspectorExamplesTest
	instanceVariableNames: 'pragmas'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Tests-Examples'!

!GTInspectorExamplesTest methodsFor: 'running' stamp: 'StefanReichhart 8/19/2016 21:19'!
allExamples
	" IMPORTANT NOTICE 
	1) We expicitly exclude examples from Roassal and Woden. Running all of those examples at once will severely slow down all tests and eventually cause segmentation faults - crash of image. This is not a fault/bug of GTExample - it happens also if the GTExample-wrapper is not used at all !! "
	^ Smalltalk gtExamplesContained reject: [ :each | 
		each ignoreTest
			or: [ each ignoreTestReport
				or: [ { 'RT'. 'RW'. 'GTDummy' } anySatisfy: [ :prefix | 
									each methodClass name beginsWith: prefix ] ] ] ]! !

!GTInspectorExamplesTest methodsFor: 'running' stamp: 'StefanReichhart 8/19/2016 21:42'!
checkAllPresentationsOf: exampleToTest
	| browser window |
	exampleToTest class name = #ProtoObject ifTrue: [ ^ self ]. " no longer implemented "
	(Pragma
		allNamed: #gtInspectorPresentationOrder:
		from: exampleToTest class
		to: ProtoObject)
		do: [ :eachPragma | 
			browser := GLMWrapper new.
			[ browser
				show: [ :composite | 
					(eachPragma methodSelector findTokens: $:) size = 1
						ifTrue: [ exampleToTest perform: eachPragma methodSelector with: composite ].
					(eachPragma methodSelector findTokens: $:) size = 2
						ifTrue: [ exampleToTest
								perform: eachPragma methodSelector
								with: composite
								with: browser ] ].
			window := browser openOn: exampleToTest ]
				on: Error
				do: [ window ifNotNil: [ window delete ].
					self
						fail:
							eachPragma method gtDisplayString , ' for '
								, exampleToTest gtDisplayString ].
			window ifNotNil: [ window delete ] ]! !

!GTInspectorExamplesTest methodsFor: 'running' stamp: 'StefanReichhart 11/14/2015 09:57'!
pragmasFor: exampleToTest
	^ pragmas 
		at: exampleToTest class 
		ifAbsentPut: [ Pragma allNamed: #gtInspectorPresentationOrder: from: exampleToTest class to: Object ]! !

!GTInspectorExamplesTest methodsFor: 'running' stamp: 'StefanReichhart 8/19/2016 12:49'!
setUp
	super setUp.
	
	pragmas := IdentityDictionary new.
	GTExampleOrganizer reset; stop! !

!GTInspectorExamplesTest methodsFor: 'running' stamp: 'StefanReichhart 8/19/2016 12:49'!
tearDown
	super tearDown.
	
	pragmas := nil.
	GTExampleOrganizer reset; start! !

!GTInspectorExamplesTest methodsFor: 'tests' stamp: 'PavelKrivanek 4/21/2017 16:11'!
testAllExamples
	| examples |
	
	self timeLimit: 15 minutes.
	
	(examples := self allExamples)
		do: [ :exampleToTest | self checkAllPresentationsOf: exampleToTest returnValue ]
		displayingProgress: 'Running ', examples size asString, ' Examples for Inspector-Presentations'! !

!GTInspectorExamplesTest methodsFor: 'tests' stamp: 'StefanReichhart 8/19/2016 21:25'!
testGlobals
	| globals |
	(globals := { Smalltalk . RBMessageNode . Trait . TClass . TBehavior . GTInspector . Object . Class . CompiledMethod })
		do: [ :each | self checkAllPresentationsOf: each ]
		displayingProgress: 'Running ', globals size asString, ' globals for Inspector-Presentations'! !

TestCase subclass: #GTSpotterProcessorsForExamplesTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Tests-Examples'!

!GTSpotterProcessorsForExamplesTest methodsFor: 'running' stamp: 'StefanReichhart 8/19/2016 12:49'!
setUp
	super setUp.
	
	GTExampleOrganizer reset; stop! !

!GTSpotterProcessorsForExamplesTest methodsFor: 'running' stamp: 'StefanReichhart 8/19/2016 12:50'!
tearDown
	super tearDown.
	
	GTExampleOrganizer reset; start! !

!GTSpotterProcessorsForExamplesTest methodsFor: 'tests' stamp: 'StefanReichhart 8/19/2016 21:15'!
testEachProcessor
	| examples |
	(examples := Smalltalk gtExamplesContained) 
		do: [ :exampleToTest |
			exampleToTest ignoreTest ifFalse: [ 
				self 
					shouldnt: [
						| step |
						(step := GTSpotterStep new)
							spotter: (GTSpotter new
								exceptionHandler: GTSpotterDebuggingExceptionHandler new; " we want to know whether one of the processors is raising an exception "
								yourself);
							origin: exampleToTest " do don't use #result here anymore, because it is not our responsibility to check whether an example runs sucessfully or not ";
							processContext: (step newContextForQuery: 'a') ] 
					raise: Error ] ]
		displayingProgress: 'Running ', examples size asString, ' Examples for GTSpotter-Processors'! !
